// -*- C++ -*-
// $RCSfile: progressbar.C,v $
// $Revision: 1.24 $
// $Author: vrc $
// $Date: 2007-09-26 22:02:22 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "common/IO/progressbar.h"
#include "common/lock.h"
#include "common/threadstate.h"
#include "common/tostring.h"
#include <iostream>
#include <pthread.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <vector>
#include <iostream>


static SLock listLock;

std::vector<Progress*> &listOfProgress() {
  static std::vector<Progress*> listPbars;
  return listPbars;
}

// Find the Progress object for this thread, but don't create a new
// one if there isn't one.  This is used only by getProgress, below,
// but doing it in a separate function allows the listLock to be
// handled more cleanly.
Progress *findProgress_() {
  ThreadID currentThread;	// identifier for *this* thread
  KeyHolder key(listLock);
  std::vector<Progress*> &list = listOfProgress();
  for(std::vector<Progress*>::size_type i=0; i<list.size(); i++)
    if(list[i]->get_thread_ID() == currentThread)
      return list[i];
  return 0;
}

// Find the Progress object for this thread, or create one if necessary.
Progress *getProgress() {
  Progress *prog = findProgress_(); // acquires and releases listLock
  if(!prog)
    prog = new Progress();	// acquires and releases listLock
  return prog;
}


bool operator==(Progress &p1, Progress &p2) {
  return p1.get_threadstate() == p2.get_threadstate();
}


Progress::Progress()
  : progress(0),
    stop(false),
    success(INCOMPLETE),
    bar_msg(" ")
{
  tState = findThreadState();
  listLock.acquire();
  listOfProgress().push_back(this);
  listLock.release();
}




Progress::~Progress() {
  remove_from_list(); //if still on list, remove yourself
}

void Progress::reset() {
  KeyHolder thekey(lockobj);
  success = INCOMPLETE;
  stop = false;
  progress = 0;
  bar_msg = " ";
}

void Progress::stop_it() {
  KeyHolder thekey(lockobj);
  stop = true;
}

bool Progress::query_stop() const {
  KeyHolder thekey(lockobj);
  return stop; 
}

void Progress::set_success() {
  KeyHolder thekey(lockobj);
  success = COMPLETE;
}

void Progress::set_failure() {
  KeyHolder thekey(lockobj);
  success = FAILED;
}

ProgressBarStatus Progress::get_success() const {
  KeyHolder thekey(lockobj);
  return success;
}

float Progress::get_progress() const {
  KeyHolder thekey(lockobj);
  return progress;  
}

void Progress::set_progress(float howfar) {
  KeyHolder thekey(lockobj);
  progress = howfar;
}

ThreadID &Progress::get_thread_ID() const {
  return tState->get_thread_ID();
}

void Progress::set_message(const std::string & msg) {
  KeyHolder thekey(lockobj);
  bar_msg = msg;
}

const std::string Progress::get_message() const {
  KeyHolder thekey(lockobj);
  return bar_msg;
// This used to be written out like this, even after other functions
// in this class were rewritten to use the KeyHolder.  Was this just
// an oversight, or is there a reason that the KeyHolder method
// doesn't work here?
//   lock();
//   std::string temp_bar_messg = bar_msg;
//   unlock();
//   return temp_bar_messg;
}

void Progress::remove_from_list() {
  listLock.acquire();
  std::vector<Progress*> &list = listOfProgress();
  for(std::vector<Progress*>::iterator i=list.begin(); i<list.end(); ++i)
    if(*i == this) {
      list.erase(i);
      break;
    }
  listLock.release();
}

void Progress::cleanup() {
  remove_from_list();
}

void Progress::make_defunct() {
  tState->make_defunct();
  remove_from_list();
  // TODO: Progress objects seem to never be destroyed.  Adding
  // "delete this" here causes them to be deleted twice, which isn't
  // good.  The Progress/ProgressObject/ProgressBar architecture needs
  // some major reworking.
//   delete this;
}
