# -*- python -*-
# $RCSfile: component.py,v $
# $Revision: 1.10 $
# $Author: vrc $
# $Date: 2007-09-26 22:02:35 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Class that represents a component of something else.  The first
# argument ('object') is the something else.  The component argument
# is a list of integers that specify which component.  The
# requirements on the given object are that it have a name() function
# that returns its name, which should be a variable known in the main
# OOF namespace, it must have an iterator() function that returns a
# IteratorP object, and it must have a __getitem__ that takes a
# list of integers and returns a Component.

# TODO LATER: Is this class needed anymore?  It originally was used to
# represent field & flux components as arguments to menu commands.
# Now those are done with separate Field and FieldIndex parameters.

from ooflib.common import debug
import types

class Component:
    def __init__(self, object, component):
        self.object = object              # Field, Flux, or Equation
        self.index = object.componenttype() # IndexP
        self.index.set(component)
    def name(self):
        return self.object.name()
    def __eq__(self, other):
        if type(other)==types.InstanceType:
            if other.__class__==self.__class__:
                return other.object==self.object and \
                           other.index.integer()==self.index.integer()
        return None
    
    def __ne__(self, other):
        return not self.__eq__(other)
        
    def __repr__(self):
        comps = self.index.components() # list of integers
        if len(comps)==0:
            return self.object.name()
        return '%s%s' % (self.object.name(), `[i+1 for i in comps]`)


